import Foundation

struct Ticket: Codable {
    let movie: String
    let seatsCount: Int
}

struct Customer: Codable {

    static var current: Customer? {
        didSet {

            // <slightly advanced comment>
            //
            // the Customer struct is immutable, so any change to their data (name, ticket) means we have to assign a completely new instance to this property; this ensures that the `didSet` handler will always be called upon change
            //
            // [naturally, the Ticket must be immutable too, which it is]
            //
            // compare this to an implementation in which the Customer struct is mutable: changing their name or ticket property directly would call the name's `didSet` or ticket's `didSet`, but not this `didSet`, because the instance itself doesn't actually change
            //
            // just a nice, bonus demonstration of how immutable data structures can help manage state in an app! (-:
            //
            // if all this mumbo-jumbo doesn't make any sense to you, don't worry; just trust that we enforce that any change to the current customer data goes through this `didSet` code


            saveCurrent() // always save the latest data so that we can restore them if the app get re-started

            if let movieTheaterId = TicketCheckObserver.shared.currentMovieTheaterId {

                // we're already inside the theater and the customer data has changed, let's see if we need to notify the theater about these changes

                if let oldCustomer = oldValue, current == nil {
                    // the customer logged out, notify the Employee apps in the respective theater to remove the customer from their list
                    postAction(.remove, customer: oldCustomer, movieTheaterId: movieTheaterId)
                }

                if let newCustomer = current {
                    // new data is available (for example, the customer logged in, or bought a ticket), send the latest data to the Employee apps in the respective theater
                    postAction(.addOrUpdate, customer: newCustomer, movieTheaterId: movieTheaterId)
                }
            }
        }
    }

    static func saveCurrent() {
        let encoder = JSONEncoder()
        let data = try? encoder.encode(Customer.current)
        UserDefaults.standard.set(data, forKey: "currentCustomer")
    }

    static func loadCurrent() {
        let decoder = JSONDecoder()
        if let data = UserDefaults.standard.data(forKey: "currentCustomer"),
           let customer = try? decoder.decode(Customer.self, from: data) {
            Customer.current = customer
        }
    }

    let name: String
    let ticket: Ticket?

    func with(_ ticket: Ticket) -> Customer {
        return Customer(name: self.name, ticket: ticket)
    }
}
