import UIKit
import CoreLocation



class OnboardingViewController: UIViewController, CLLocationManagerDelegate {

    // the "skip" button on this screen performs the "DismissOnboarding" segue directly, which moves forward to the login screen

    let locationManager = CLLocationManager()

    // the "allow location access" button performs this action, where we invoke the "allow access?" system dialog; the result will be passed by iOS to the "didChangeAuthorizationStatus" delegete below
    @IBAction func allowLocationAccess(_ sender: Any) {
        locationManager.delegate = self
        locationManager.requestAlwaysAuthorization()
    }

    func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {

        // whatever the user's decision (allow always, allow while in use, deny), we proceed to the login screen
        //
        // - "always" access = the app will have full functionality
        //
        // - "while in use" access = the app will only forward their ticket to the employee's app if it's open, but not in the background; this means limited functionality, but the app is still usable (they can open the app and hold the smartphone in their hand, and the ticket-check employee will still see their ticket in their own app)
        //
        // - "deny" access = the app won't forward the ticket at all (because it doesn't know the location, so it doesn't know where to forward it), which means even more limited functionality, but also still usable (maybe the customer only wants to use the app to buy tickets, and simply show their smartphone screen to the ticket-check employee)
        //
        // generally speaking, that's a good pattern to follow: degrade gracefully, your app should still be usable without location access

        performSegue(withIdentifier: "DismissOnboarding", sender: nil)
    }
}



class LoginViewController: UIViewController {

    @IBOutlet weak var nameTextField: UITextField!

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        nameTextField.text = ""
        nameTextField.becomeFirstResponder()
    }

    // this segue is in response to the "log in" button
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        let name = nameTextField.text! != "" ? nameTextField.text! : "Anonymous"

        Customer.current = Customer(name: name, ticket: nil)
    }

    @IBAction func unwindToLogin(sender: UIStoryboardSegue) {

        // the "start over" button on the "buy tickets" and "your tickets" screens leads here, that is, back to the login screen; so we also clear the currently logged-in customer's data

        Customer.current = nil
    }
}



class BuyTicketsViewController: UIViewController {

    @IBOutlet weak var nameLabel: UILabel!

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        nameLabel.text = "Hi, \(Customer.current!.name)!"
    }

    // this segue is in response to the "buy tickets" button
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {

        // let's just randomly select a movie + how many seats

        let movies = ["Ready Player One", "Tomb Raider", "Black Panther"]
        let randomMovie = movies[Int(arc4random_uniform(3))]
        let randomSeats = Int(arc4random_uniform(4)) + 1

        let ticket = Ticket(movie: randomMovie, seatsCount: randomSeats)
        Customer.current = Customer.current!.with(ticket)
    }
}



class TicketsViewController: UIViewController {

    @IBOutlet weak var nameLabel: UILabel!
    @IBOutlet weak var ticketsLabel: UILabel!

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        nameLabel.text = "Hi, \(Customer.current!.name)!"

        let ticket = Customer.current!.ticket!
        ticketsLabel.text = "\(ticket.seatsCount) x \(ticket.movie)"
    }
}
